import React, { useState, useCallback } from 'react';
import { utils } from '@ray-js/panel-sdk';
import { useAtomValue, useSetAtom } from 'jotai';
import { ColorData, colorDataAtom, selectColorDataAtom } from '@/atoms';
import { View, Text, Slider, Image, usePageEvent } from '@ray-js/ray';
import styles from './index.module.less';
import { Props } from './index.type';
import imgs from '../../res';

const { hsv2rgbString } = utils;

// 平移法
const HubColorCircle: React.FC<Props> = props => {
  const dpState = props.DpStateData;
  const codeMap = props.DpCodesMap;

  const setColorStateAtom = useSetAtom(colorDataAtom);
  const colorStateInAtom = useAtomValue(selectColorDataAtom);

  const data = {
    radius: 135, // 容器半径
    innerRadius: 76, // 内半径
    thumbRadius: 27.5, // 圆球半径
    thumbInnerRadius: 22.5, // 圆球内环
    ringSize: 135 - 76, // 圆环尺寸

    colorThumRadius: 76 - 5,

    cx: 135 - 27.5, // 圆心相对父容器位置
    cy: 135 - 27.5, // 圆心相对父容器位置

    fixedLength: 135 - (135 - 76) * 0.5, // 可拖动圆球至原点的固定距离（令圆球始终在在色环中居中）

    containerOffsetX: 135 - 27.5,
    containerOffsetY: (135 - 76) * 0.5 - 27.5 * 0.5,

    mouseMove: {
      // 父元素偏移
      x: 0,
      y: 0,
    },

    location: {
      // 点击时的小球位置
      x: 0,
      y: 0,
    },

    disabled: false,
    RingBackground: imgs.colorBg,
    hue: 90, // 色相
  };

  const atomState = {
    // 父元素偏移
    mouseMoveX: 0,
    mouseMoveY: 0,

    // 点击时的小球位置
    locationX: 0,
    locationY: 0,
  };

  const hueToColor = hue => {
    return hsv2rgbString(hue, 1000, 1000);
  };

  const [translateX, setTranslateX] = useState(0);
  const [translateY, setTranslateY] = useState(0);
  const [nowColor, setNowColor] = useState(hueToColor(data.hue));
  const [hueState, sethueState] = useState(data.hue);

  const getCoordByHue = (hue: number) => {
    const rad = ((360 - hue) * Math.PI) / 180;
    const x = data.cx + data.fixedLength * Math.cos(rad);
    const y = data.cy + data.fixedLength * Math.sin(rad);
    return { x, y };
  };

  const handleBarTouch = e => {
    atomState.locationX = e.origin.currentTarget.offsetLeft;
    atomState.locationY = e.origin.currentTarget.offsetTop;
    setColorStateAtom(atomState);

    console.log(data.location, 'touch');
    atomState.mouseMoveX = e.origin.currentTarget.offsetLeft;
    atomState.mouseMoveY = e.origin.currentTarget.offsetTop;
  };

  const getHueByCoord = (dx: number, dy: number) => {
    // 0 ~ 2π
    const rad = getRadianByCoord(dx, dy);
    return (rad * 180) / Math.PI;
  };

  const getRadianByCoord = (dx: number, dy: number) => {
    const { thumbRadius } = data;
    // 相对中心点的坐标
    const xCenter = dx - data.cx - thumbRadius;
    const yCenter = dy - data.cy - thumbRadius;

    let rad = Math.atan2(yCenter, xCenter);
    if (xCenter > 0 && yCenter > 0) rad = Math.PI * 2 - rad;
    if (xCenter < 0 && yCenter > 0) rad = Math.PI * 2 - rad;
    if (xCenter < 0 && yCenter < 0) rad = Math.abs(rad);
    if (xCenter > 0 && yCenter < 0) rad = Math.abs(rad);
    if (xCenter === 0 && yCenter > 0) rad = (Math.PI * 3) / 2;
    if (xCenter === 0 && yCenter < 0) rad = Math.PI / 2;
    return rad;
  };

  const handleMove = e => {
    // 最近一次的移动路程，dx的正负决定了移动的正负

    const { locationX, locationY } = colorStateInAtom;
    const dx = e.origin.touches[0].clientX - locationX;
    const dy = e.origin.touches[0].clientY - locationY;

    const hue = Math.round(getHueByCoord(dx, dy));
    const { x = 0, y = 0 } = getCoordByHue(hue);

    const color = hueToColor(hue);
    setTranslateX(x);
    setTranslateY(y);
    setNowColor(color);
    sethueState(hue);
  };

  return (
    <View className={styles.header}>
      <View
        className={styles.circleContent}
        style={{ width: `${data.radius * 2}px`, height: `${data.radius * 2}px` }}
      >
        {/* 圆环 */}
        <View
          className={styles.sectionRing}
          style={{ width: `${data.radius * 2}px`, height: `${data.radius * 2}px` }}
        >
          <Image
            style={{
              width: `${data.radius * 2}px`,
              height: `${data.radius * 2}px`,
              borderRadius: '50%',
            }}
            src={imgs.colorBg}
          />
        </View>

        {/* 圆环中间色彩值 */}
        <View
          className={styles.colorThum}
          style={{
            position: 'absolute',
            left: `${data.radius - data.colorThumRadius}px`,
            top: `${data.radius - data.colorThumRadius}px`,
            width: `${data.colorThumRadius * 2}px`,
            height: `${data.colorThumRadius * 2}px`,
            borderRadius: '50%',
            backgroundColor: nowColor,
          }}
        >
          Angle:{hueState}°
        </View>

        {/* 圆球 */}
        <View
          className={styles.bar}
          style={{
            top: `${data.containerOffsetY}px`,
            left: `${data.containerOffsetX}px`,
            width: `${data.thumbRadius * 2}px`,
            height: `${data.thumbRadius * 2}px`,
            borderRadius: '50%',
            opacity: 1,
            transform: `translateX(${translateX}px) translateY(${translateY}px)`,
          }}
          onTouchStart={handleBarTouch}
          onTouchMove={handleMove}
        >
          <View
            style={{
              width: `${data.thumbInnerRadius * 2}px`,
              height: `${data.thumbInnerRadius * 2}px`,
              borderRadius: '50%',
              backgroundColor: nowColor,
            }}
          />
        </View>
      </View>
    </View>
  );
};

export default HubColorCircle;
